﻿using ComPDFKit.PDFAnnotation.Form;
using Compdfkit_Tools.Measure;
using ComPDFKitViewer.AnnotEvent;
using ComPDFKitViewer.PdfViewer;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Compdfkit_Tools.PDFControl
{
    /// <summary>
    /// CPDFMeasureBarControl.xaml 的交互逻辑
    /// </summary>
    public partial class CPDFMeasureBarControl : UserControl
    {
        #region Data

        private bool isFirstLoad = true;

        readonly Dictionary<string, string> buttonDict = new Dictionary<string, string>
        {
            {"Straightness","M18.2782 5.75646L14.7427 2.22093L13.5052 3.45837L15.9802 5.93333L14.9195 6.99399L12.4446 4.51903L10.6768 6.28679L13.1518 8.76175L12.0911 9.82241L9.61615 7.34745L7.84838 9.11522L10.3233 11.5902L9.26268 12.6508L6.78772 10.1759L5.01996 11.9436L7.49492 14.4186L6.43426 15.4793L3.9593 13.0043L2.72186 14.2417L6.25739 17.7773L18.2782 5.75646ZM15.8033 1.16027L14.7427 0.0996094L13.682 1.16027L1.6612 13.1811L0.600539 14.2417L1.6612 15.3024L5.19673 18.8379L6.25739 19.8986L7.31805 18.8379L19.3389 6.81712L20.3995 5.75646L19.3389 4.6958L15.8033 1.16027Z"},
            {"Multiline", "M1 3C1 1.89543 1.89543 1 3 1C3.83934 1 4.55793 1.51704 4.85462 2.25H16.1454C16.4421 1.51704 17.1607 1 18 1C19.1046 1 20 1.89543 20 3C20 3.83934 19.483 4.55793 18.75 4.85462V17V17.75H18H14.2011L15.0167 18.5765L14.0138 19.5928L11.9986 17.5505L11.4972 17.0423L11.9986 16.5342L14.0138 14.4918L15.0167 15.5082L14.2847 16.25H17.25V4.85462C16.7487 4.65168 16.3483 4.25135 16.1454 3.75H4.85462C4.65168 4.25135 4.25135 4.65168 3.75 4.85462V16.25H6.75V15H8.25V19H6.75V17.75H3H2.25V17V4.85462C1.51704 4.55793 1 3.83934 1 3Z"},
            {"Polygonal","M1.87111 7.6963L10.5 1.42705L19.1289 7.6963L15.8329 17.8402H5.16705L1.87111 7.6963Z"},
            {"Rectangles","M20.25 2.25H0.75V17.75H20.25V2.25ZM18.75 10.75V16.25H2.25V10.75H18.75ZM18.75 9.25V3.75H2.25V9.25H18.75ZM6.5 5.5H3.5V7.5H6.5V5.5ZM3.5 12.5H6.5V14.5H3.5V12.5Z"}
        };


        private CPDFViewer pdfViewer;

        private MeasurePropertyControl measurePropertyControl = null;

        private enum MeasureType
        {
            UnKnown = -1,
            Straightness,
            Multiline,
            Polygonal,
            Rectangles
        }
        private MeasureType StringToType(string type)
        {
            switch (type)
            {
                case "Straightness":
                    return MeasureType.Straightness;
                case "Multiline":
                    return MeasureType.Multiline;
                case "Polygonal":
                    return MeasureType.Polygonal;
                case "Rectangles":
                    return MeasureType.Rectangles;
                default:
                    break;
            }
            return MeasureType.UnKnown;
        }

        #endregion

        #region Create Default UI

        public CPDFMeasureBarControl()
        {
            InitializeComponent();
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            if (isFirstLoad)
            {
                foreach (KeyValuePair<string, string> data in buttonDict)
                {
                    string Path = data.Value;
                    string name = data.Key;

                    Geometry annotationGeometry = Geometry.Parse(Path);
                    Path path = new Path
                    {
                        Width = 20,
                        Height = 20,
                        Data = annotationGeometry,
                        Fill = new SolidColorBrush(Color.FromRgb(0x43, 0x47, 0x4D)),
                    };
                    if (name == "Polygonal")
                    {
                        path.Fill = null;
                        path.Stroke = new SolidColorBrush(Color.FromRgb(0x43, 0x47, 0x4D));
                        path.StrokeThickness = 1.5;
                    }
                    else if (name == "Rectangles")
                    {
                        path.Fill = null;
                        path.Data = new RectangleGeometry(new Rect(3.25, 2.75, 14.5, 14.5));
                        path.Stroke = new SolidColorBrush(Color.FromRgb(0x43, 0x47, 0x4D));
                        path.StrokeThickness = 1.5;
                    }
                    CreateButtonForPath(path, name);
                }
                isFirstLoad = false;
            }
        }

        private void CreateButtonForPath(Path path, String name)
        {
            StackPanel stackPanel = new StackPanel();
            TextBlock textBlock = new TextBlock();
            if (path != null)
            {
                stackPanel.Orientation = Orientation.Horizontal;
                stackPanel.HorizontalAlignment = HorizontalAlignment.Center;
                stackPanel.Children.Add(path);
            }
            if (!string.IsNullOrEmpty(name))
            {
                textBlock.Text = name;
                textBlock.VerticalAlignment = VerticalAlignment.Center;
                textBlock.Margin = new Thickness(8, 0, 0, 0);
                textBlock.FontSize = 12;

                stackPanel.Children.Add(textBlock);
            }

            Style style = (Style)FindResource("RoundMarginToggleButtonStyle");
            ToggleButton button = new ToggleButton();
            button.BorderThickness = new Thickness(0);
            button.Padding = new Thickness(10, 5, 10, 5);
            button.Tag = name;
            button.ToolTip = name;
            button.Style = style;
            button.Content = stackPanel;
            button.Click += MeasureBtn_Click;
            MeasureGrid.Children.Add(button);
        }

        #endregion

        #region Even Process

        public void InitWithPDFViewer(CPDFViewer pdfViewer, MeasurePropertyControl FromProperty)
        {
           
            if(this.pdfViewer!=null)
            {
                this.pdfViewer.AnnotActiveHandler -= PdfViewer_AnnotActiveHandler;
            }
            this.pdfViewer = pdfViewer;
            pdfViewer.AnnotActiveHandler -= PdfViewer_AnnotActiveHandler;
            pdfViewer.AnnotActiveHandler += PdfViewer_AnnotActiveHandler;
            measurePropertyControl = FromProperty;
        }

        private void PdfViewer_AnnotActiveHandler(object sender, AnnotAttribEvent e)
        {
            if (e != null)
            {
                if(e.IsAnnotCreateReset)
                {
                    List<AnnotHandlerEventArgs> argsList = e.GetAnnotHandlerEventArgs(e.GetAnnotTypes());
                    measurePropertyControl.SetPropertyForMeasureCreate(argsList[0]);
                }
                else
                {
                    measurePropertyControl.SetPorpertyForMeasureModify(e);
                }
            }
        }

        public void ClearAllToolState()
        {
            foreach (UIElement child in MeasureGrid.Children)
            {
                if (child is ToggleButton toggle)
                {
                    toggle.IsChecked = false;
                }
            }
        }

        private void ClearToolState(UIElement sender)
        {
            foreach (UIElement child in MeasureGrid.Children)
            {
                if (child is ToggleButton toggle && (child as ToggleButton) != (sender as ToggleButton))
                {
                    toggle.IsChecked = false;
                }
            }
        }

        private void MeasureBtn_Click(object sender, RoutedEventArgs e)
        {
            ClearToolState(sender as ToggleButton);
            measurePropertyControl.SetPropertyForMeasureCreate(null);
            if ((bool)(sender as ToggleButton).IsChecked)
            {
                switch (StringToType((sender as ToggleButton).Tag.ToString()))
                {
                    case MeasureType.UnKnown:
                        break;
                    case MeasureType.Straightness:
                        CreateStraightness();
                        break;
                    case MeasureType.Multiline:
                        CreateMultiline();
                        break;
                    case MeasureType.Polygonal:
                        CreatePolygonal();
                        break;
                    case MeasureType.Rectangles:
                        CreateRectangles();
                        break;
                    default:
                        break;
                }
            }
            else
            {
                pdfViewer.SetMouseMode(MouseModes.PanTool);
            }
        }

        #endregion

        #region Create Form

        private void CreateStraightness()
        {
            LineMeasureArgs lineMeasureArgs = new LineMeasureArgs();
            lineMeasureArgs.LineColor = Colors.Red;
            lineMeasureArgs.LineWidth = 2;
            lineMeasureArgs.Transparency = 1;
            lineMeasureArgs.FontColor= Colors.Red;
            lineMeasureArgs.FontName = "Arial";
            lineMeasureArgs.FontSize = 14;
            pdfViewer?.ClearSelectAnnots();
            pdfViewer?.SetMouseMode(MouseModes.AnnotCreate);
            pdfViewer?.SetToolParam(lineMeasureArgs);
            measurePropertyControl.SetPropertyForMeasureCreate(lineMeasureArgs);
        }

        private void CreateMultiline()
        {

        }

        private void CreatePolygonal()
        {

        }

        private void CreateRectangles()
        {

        }

        #endregion


    }
}
